<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;

// controller criado para as telas da issue 520239 que foi cancelada
class ExportacaoIntegracaoTxt extends \Zeedhi\Framework\Controller\Report {
//   protected $crudService;
// 	protected $parameterBag;
// 	protected $dataSourceName;
	protected $connection;
	protected $environment;
	protected $entityManager;
	protected $birtStrategy;
	protected $utilities;
	
	protected $reportMapping = array (
    'PME02600' => 
      array (
          'strategy' => 'BIRT',
          'parameterMapping' => 
          array (
            "CENTRAL" => "CDFILCENTPME",
          )
      )
    );

    public function __construct(ReportService $reportService, $connection, $entityManager, $birtStrategy, $utilities) {
        // parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->birtStrategy  = $birtStrategy;
        $this->utilities   = $utilities;
        parent::__construct($reportService);
	}
	
	public function exportIntegrationTxt(DTO\Request\Row $request, DTO\Response $response) {
		$this->connection->beginTransaction();
		try {
		    $row = $request->getRow();
		    $action = $row['action'];
		    $strResult = '';
		    if ($action === 'cardapio') {
		        $linhasCodigo1 = $this->getCardapioFirstLine($row);
		        foreach ($linhasCodigo1 as $linha) {
	                $strResult .= $linha['LINHA']."\n";
		            $pratos = $this->getPratosSecondLine($row['CDFILCENTPME'], $linha['NRCARDPME'], $linha['NRCARDIAPME']);
		            foreach($pratos as $prato) {
		                $strResult .= "02;".$prato['CDPRATO']."\n";
		            }
		            $strResult .= "03;".$linha['DTCONSUMO']."\n";
		        }
		        
		    } else if ($action === 'servico') {
		        $linhas = $this->getServicoLines($row);
		        foreach ($linhas as $linha) {
		            $linhaText = $linha['NRCONTINTPME'].$linha['CDFILIAL'].$linha['TIPO_REFEICAO'].$linha['DIETA'].$linha['MODALIDADE'].$linha['FAIXA_ETARIA'].$linha['DTCONSUMO'];
		            $pratos = $this->getPratosSecondLine($row['CDFILCENTPME'], $linha['NRCARDPME'], $linha['NRCARDIAPME']);   
		            if (!empty($pratos)) $linhaText .= str_pad(count($pratos), 10, '0', STR_PAD_LEFT);
		            else $linhaText .= str_pad(0, 10, '0', STR_PAD_LEFT);
		            $strResult .= $linhaText."\n";
		        }
		    }
		    $result = $this->generateFile($action, $strResult);
		    // $this->connection->commit();
		    $response->addDataSet(new DataSet("export", $result));
		    
		} catch (\Exception $e) {
		    $this->connection->rollback();
		    $response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function generateFile($action, $content) {
        // $tmpName = tempnam(sys_get_temp_dir(), $action);
        $tmpName = sys_get_temp_dir()."/".$action.".txt";
        $file = fopen($tmpName, 'w');
        
        fwrite($file, $content);
        fclose($file);
        return array('content' => base64_encode(file_get_contents($tmpName)));
	}
	
	private function getCardapioFirstLine($row) {
	    $sql = "SELECT F.NRCONTINTPME, MODALIDADE.CDINTEGRA AS MODALIDADE, TIPO_REFEICAO.CDINTEGRA AS TIPO_REFEICAO, DIETA.CDINTEGRA AS DIETA, FAIXA_ETARIA.CDINTEGRA AS FAIXA_ETARIA, GRUPO_CARDAPIO.CDINTEGRA AS GRUPO_CARDAPIO,
                    '01;' || F.NRCONTINTPME || ';' || MODALIDADE.CDINTEGRA || ';' || TIPO_REFEICAO.CDINTEGRA || ';' || DIETA.CDINTEGRA || ';' || FAIXA_ETARIA.CDINTEGRA || ';' || GRUPO_CARDAPIO.CDINTEGRA AS LINHA,
                    TO_CHAR(D.DTCARDIAPME, 'DDMMYYYY') AS DTCONSUMO, D.NRCARDPME, D.NRCARDIAPME
                    FROM CARDAPIOPME C
                    INNER JOIN CARDIAPME D ON D.CDFILCENTPME = C.CDFILCENTPME 
                      AND D.NRCARDPME = C.NRCARDPME 
                      AND (
                        ((:CDFILIAL = 'T') OR INSTR(:CDFILIAL, D.CDFILIAL) > 0)
                        OR D.CDFILIAL = D.CDFILCENTPME
                      ) 
                      AND D.DTCARDIAPME BETWEEN :DTINI 
                      AND :DTFIM 
                    INNER JOIN FILIAL F ON F.CDFILIAL = D.CDFILIAL AND F.NRCONTINTPME IS NOT NULL
                    INNER JOIN TIPOCARD TPC ON TPC.CDFILCENTPME = D.CDFILCENTPME AND TPC.CDTIPOCARD = C.CDTIPOCARD AND TPC.NRINTEGRACARDPME IS NOT NULL AND TPC.IDTPCARDATIV = 'S'
                    INNER JOIN INTEGRACARDPME FAIXA_ETARIA ON FAIXA_ETARIA.NRINTEGRACARDPME = TPC.NRINTEGRACARDPME AND FAIXA_ETARIA.IDTIPO = 3
                    INNER JOIN GRUPTIPOCARD GTPC ON GTPC.CDFILCENTPME = D.CDFILCENTPME AND GTPC.CDGRUPTIPOCARD = TPC.CDGRUPTIPOCARD AND GTPC.NRINTEGRACARDPME IS NOT NULL
                    INNER JOIN INTEGRACARDPME MODALIDADE ON MODALIDADE.NRINTEGRACARDPME = GTPC.NRINTEGRACARDPME AND MODALIDADE.IDTIPO = 1
                    INNER JOIN SERVICO S ON (S.CDFILIAL = D.CDFILIAL OR S.CDFILIAL = D.CDFILCENTPME) AND S.CDSERVFILI = D.CDSERVFILI AND S.NRINTEGRACARDPME IS NOT NULL
                    INNER JOIN INTEGRACARDPME DIETA ON DIETA.NRINTEGRACARDPME = S.NRINTEGRACARDPME AND DIETA.IDTIPO = 4
                    INNER JOIN GRUPSERV GS ON GS.CDGRUPSERV = S.CDGRUPSERV AND GS.NRINTEGRACARDPME IS NOT NULL
                    INNER JOIN INTEGRACARDPME TIPO_REFEICAO ON TIPO_REFEICAO.NRINTEGRACARDPME = GS.NRINTEGRACARDPME AND TIPO_REFEICAO.IDTIPO = 2
                    INNER JOIN ROTATRANS R ON (R.CDFILIAL = D.CDFILIAL OR R.CDFILIAL = D.CDFILCENTPME) AND R.NRINTEGRACARDPME IS NOT NULL
                    INNER JOIN INTEGRACARDPME GRUPO_CARDAPIO ON GRUPO_CARDAPIO.NRINTEGRACARDPME = R.NRINTEGRACARDPME AND GRUPO_CARDAPIO.IDTIPO = 5
                     WHERE C.CDFILCENTPME = :CDFILCENTPME
                       AND (C.DTINICARDPME BETWEEN :DTINI AND :DTFIM
                       OR C.DTFINCARDPME BETWEEN :DTINI AND :DTFIM
                       OR :DTINI BETWEEN C.DTINICARDPME AND C.DTFINCARDPME
                       OR :DTFIM BETWEEN C.DTINICARDPME AND C.DTFINCARDPME)
                       AND ((:CDTIPOCARD = 'T') OR INSTR(:CDTIPOCARD, C.CDTIPOCARD) > 0)
                       AND ((:CDGRUPTIPOCARD = 'T') OR INSTR(:CDGRUPTIPOCARD, GTPC.CDGRUPTIPOCARD) > 0)
                       AND ((:CDSERVFILI = 'T') OR INSTR(:CDSERVFILI, D.CDSERVFILI) > 0)
                       AND ((:CDGRUPSERV = 'T') OR INSTR(:CDGRUPSERV, GS.CDGRUPSERV) > 0)
                       AND EXISTS (SELECT CDPRATO FROM COMPCARDPME CC WHERE CC.CDFILCENTPME = D.CDFILCENTPME AND CC.NRCARDPME = D.NRCARDPME AND CC.NRCARDIAPME = D.NRCARDIAPME)
                       ";
        $params = array(
            "DTINI" => $row['DTDE'],
            "DTFIM" => $row['DTATE'],
            "CDFILCENTPME" => $row['CDFILCENTPME'],
            "CDFILIAL" => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "CDTIPOCARD" => $row['CDTIPOCARD'],
            "CDGRUPSERV" => $row['CDGRUPSERV'],
            "CDGRUPTIPOCARD" => $row['CDGRUPTIPOCARD'],
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $params);
	}
	
	private function getPratosSecondLine($cdfilcentpme, $nrcadpme, $nrcadiapme) {
	    $sql = "SELECT NVL(CDPRATEXTPME, P.CDPRATO) CDPRATO 
	            FROM COMPCARDPME P 
	            JOIN PRATO PT ON P.CDPRATO = PT.CDPRATO AND P.CDFILAUXPRAT = PT.CDFILAUXPRAT
	            WHERE P.CDFILCENTPME = :CDFILCENTPME 
                  AND P.NRCARDIAPME = :NRCARDIAPME 
                  AND P.NRCARDPME = :NRCARDPME";
        $params = array(
            'CDFILCENTPME' => $cdfilcentpme,
            'NRCARDPME' => $nrcadpme,
            'NRCARDIAPME' => $nrcadiapme,
        );
        return $this->entityManager->getConnection()->fetchAll($sql, $params);
	}
	
	private function getServicoLines($row) {
	    $sql = "SELECT LPAD(F.CDFILIAL, 10, '0') CDFILIAL, LPAD(F.NRCONTINTPME, 10, '0') NRCONTINTPME, LPAD(MODALIDADE.CDINTEGRA, 10, '0') AS MODALIDADE, 
	            LPAD(TIPO_REFEICAO.CDINTEGRA, 10, '0') AS TIPO_REFEICAO, LPAD(DIETA.CDINTEGRA, 10, '0') AS DIETA, 
	            LPAD(FAIXA_ETARIA.CDINTEGRA, 10, '0') AS FAIXA_ETARIA, LPAD(GRUPO_CARDAPIO.CDINTEGRA, 10, '0') AS GRUPO_CARDAPIO,
                    TO_CHAR(D.DTCARDIAPME, 'DDMMYYYY') AS DTCONSUMO, D.NRCARDPME, D.NRCARDIAPME
                    FROM CARDAPIOPME C
                    INNER JOIN CARDIAPME D ON D.CDFILCENTPME = C.CDFILCENTPME 
                      AND D.NRCARDPME = C.NRCARDPME 
                      AND (
                        ((:CDFILIAL = 'T') OR INSTR(:CDFILIAL, D.CDFILIAL) > 0)
                        OR D.CDFILIAL = D.CDFILCENTPME
                      ) 
                      AND D.DTCARDIAPME BETWEEN :DTINI 
                      AND :DTFIM 
                    INNER JOIN FILIAL F ON F.CDFILIAL = D.CDFILIAL AND F.NRCONTINTPME IS NOT NULL
                    INNER JOIN TIPOCARD TPC ON TPC.CDFILCENTPME = D.CDFILCENTPME AND TPC.CDTIPOCARD = C.CDTIPOCARD AND TPC.NRINTEGRACARDPME IS NOT NULL AND TPC.IDTPCARDATIV = 'S'
                    INNER JOIN INTEGRACARDPME FAIXA_ETARIA ON FAIXA_ETARIA.NRINTEGRACARDPME = TPC.NRINTEGRACARDPME AND FAIXA_ETARIA.IDTIPO = 3
                    INNER JOIN GRUPTIPOCARD GTPC ON GTPC.CDFILCENTPME = D.CDFILCENTPME AND GTPC.CDGRUPTIPOCARD = TPC.CDGRUPTIPOCARD AND GTPC.NRINTEGRACARDPME IS NOT NULL
                    INNER JOIN INTEGRACARDPME MODALIDADE ON MODALIDADE.NRINTEGRACARDPME = GTPC.NRINTEGRACARDPME AND MODALIDADE.IDTIPO = 1
                    INNER JOIN SERVICO S ON (S.CDFILIAL = D.CDFILIAL OR S.CDFILIAL = D.CDFILCENTPME) AND S.CDSERVFILI = D.CDSERVFILI AND S.NRINTEGRACARDPME IS NOT NULL
                    INNER JOIN INTEGRACARDPME DIETA ON DIETA.NRINTEGRACARDPME = S.NRINTEGRACARDPME AND DIETA.IDTIPO = 4
                    INNER JOIN GRUPSERV GS ON GS.CDGRUPSERV = S.CDGRUPSERV AND GS.NRINTEGRACARDPME IS NOT NULL
                    INNER JOIN INTEGRACARDPME TIPO_REFEICAO ON TIPO_REFEICAO.NRINTEGRACARDPME = GS.NRINTEGRACARDPME AND TIPO_REFEICAO.IDTIPO = 2
                    INNER JOIN ROTATRANS R ON (R.CDFILIAL = D.CDFILIAL OR R.CDFILIAL = D.CDFILCENTPME) AND R.NRINTEGRACARDPME IS NOT NULL
                    INNER JOIN INTEGRACARDPME GRUPO_CARDAPIO ON GRUPO_CARDAPIO.NRINTEGRACARDPME = R.NRINTEGRACARDPME AND GRUPO_CARDAPIO.IDTIPO = 5
                     WHERE C.CDFILCENTPME = :CDFILCENTPME
                       AND (C.DTINICARDPME BETWEEN :DTINI AND :DTFIM
                       OR C.DTFINCARDPME BETWEEN :DTINI AND :DTFIM
                       OR :DTINI BETWEEN C.DTINICARDPME AND C.DTFINCARDPME
                       OR :DTFIM BETWEEN C.DTINICARDPME AND C.DTFINCARDPME)
                       AND ((:CDTIPOCARD = 'T') OR INSTR(:CDTIPOCARD, C.CDTIPOCARD) > 0)
                       AND ((:CDGRUPTIPOCARD = 'T') OR INSTR(:CDGRUPTIPOCARD, GTPC.CDGRUPTIPOCARD) > 0)
                       AND ((:CDSERVFILI = 'T') OR INSTR(:CDSERVFILI, D.CDSERVFILI) > 0)
                       AND ((:CDGRUPSERV = 'T') OR INSTR(:CDGRUPSERV, GS.CDGRUPSERV) > 0)
                       AND EXISTS (SELECT CDPRATO FROM COMPCARDPME CC WHERE CC.CDFILCENTPME = D.CDFILCENTPME AND CC.NRCARDPME = D.NRCARDPME AND CC.NRCARDIAPME = D.NRCARDIAPME)
                       ";
        $params = array(
            "DTINI" => $row['DTDE'],
            "DTFIM" => $row['DTATE'],
            "CDFILCENTPME" => $row['CDFILCENTPME'],
            "CDFILIAL" => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "CDTIPOCARD" => $row['CDTIPOCARD'],
            "CDGRUPSERV" => $row['CDGRUPSERV'],
            "CDGRUPTIPOCARD" => $row['CDGRUPTIPOCARD'],
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $params);
	}
	
	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
      $row = $request->getRow();
    //   $nrorg = $this->utilities->getNrOrg();
      
    //   $row['NRORG'] = $nrorg;
      $this->birtStrategy->setReportLocale($this->utilities->getLanguage());
      $this->birtStrategy->setReportFormat(strtolower($row['reportFormat']));
    }
}