import {
  Metadata, Datasource, IEventParam, Loader, IDatasource,
} from '@zeedhi/core';
import {
  GridEditable, Card, Select, Button, ITabsBeforeChangeEventParam, Tabs,
} from '@zeedhi/common';

export class Exp02100RotaEntregaController {
  public detailIsVisible = false;
  
  private gridEditController = Loader.getInstance('GridEditableController');
  
  private utilities = Loader.getInstance('Utilities');
  
  private crudController = Loader.getInstance('CrudController');
  
  private appController = Loader.getInstance('AppController');
  
  public selectedRotaEntrega = '';
  
  get gridColsReturnValue(): number {
    return this.detailIsVisible ? 6 : 12;
  }
  
  public onChangeRotaCurrentRow({ component }: IEventParam<Datasource>) {
    const cdrota = component.currentRow.CDROTATRANS;
    const gridRotaEntrega: GridEditable = Metadata.getInstance('gridRotaEntrega');
    const isEditing = gridRotaEntrega.editing;
    
    if (cdrota && !isEditing) {
      const gridMunicipioRota = Metadata.getInstance<GridEditable>('gridMunicipioRota');
      this.selectedRotaEntrega = `${component.currentRow.CDROTATRANS} - ${component.currentRow.NMROTATRANS}`;
      
      const conditions = [
        {
          name: 'CDROTATRANS',
          operator: '=',
          value: component.currentRow.CDROTATRANS,
        },
      ];
      
      gridMunicipioRota.datasource.filter = conditions;
      gridMunicipioRota.datasource.get();
      
      // this.cardProdutoVisible();
    } else {
      this.selectedRotaEntrega = `${component.currentRow.CDROTATRANS} - ${component.currentRow.NMROTATRANS}`;
      this.closeCardDetail();
    }
  }
  
  public setFilterGridOnMounted({ component }: IEventParam<GridEditable>) {
    const gridRotaEntrega: GridEditable = Metadata.getInstance('gridRotaEntrega');
    const conditions = [
      {
        name: 'CDROTATRANS',
        operator: '=',
        value: gridRotaEntrega?.datasource?.currentRow?.CDROTATRANS || '',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public closeCardDetail() {
    const card = Metadata.getInstance<Card>('cardDetail');
    card.isVisible = false;
    this.detailIsVisible = false;
  }
  
  private cardDetailVisible() {
    const card = Metadata.getInstance<Card>('cardDetail');
    
    try {
      if (!this.gridEditController.newRow) {
        card.isVisible = true;
        this.detailIsVisible = true;
      }
    } catch (Error) {
      card.isVisible = false;
      this.detailIsVisible = false;
    }
  }
  
  public reloadAndCloseCardDetail() {
    const gridRotaEntrega = Metadata.getInstance<GridEditable>('gridRotaEntrega');
    this.gridEditController.reload({ component: gridRotaEntrega });
    this.closeCardDetail();
  }
  
  public clearFieldsAndSetFilter({ component }: IEventParam<Select>) {
    const estado = document.getElementsByName('SGESTADO_editable_').length > 0
      ? Metadata.getInstance<Select>('SGESTADO_editable_')
      : undefined;
    const municipio = document.getElementsByName('CDMUNICIPIO_editable_').length > 0
      ? Metadata.getInstance<Select>('CDMUNICIPIO_editable_')
      : undefined;
    const bairro = document.getElementsByName('CDBAIRRO_editable_').length > 0
      ? Metadata.getInstance<Select>('CDBAIRRO_editable_')
      : undefined;
    if (estado && municipio && bairro) {
      if (component?.name?.includes('CDPAIS') && component.value) {
        // estado.datasource.addFilter('CDPAIS', component.value);
        estado.datasource.filter = this.utilities.addFilter(estado?.datasource, 'CDPAIS', '=', component.value);
        estado.value = '';
        // municipio.datasource.addFilter('CDPAIS', component.value);
        municipio.datasource.filter = this.utilities.addFilter(municipio?.datasource, 'CDPAIS', '=', component.value);
        municipio.value = '';
        // bairro.datasource.addFilter('CDPAIS', component.value);
        bairro.datasource.filter = this.utilities.addFilter(bairro?.datasource, 'CDPAIS', '=', component.value);
        bairro.value = '';
        estado.datasource.get();
        municipio.datasource.get();
        bairro.datasource.get();
      } else if (component?.name?.includes('SGESTADO') && component.value) {
        // municipio.datasource.addFilter('SGESTADO', component.value);
        municipio.datasource.filter = this.utilities.addFilter(municipio?.datasource, 'SGESTADO', '=', component.value);
        municipio.value = '';
        // bairro.datasource.addFilter('SGESTADO', component.value);
        bairro.datasource.filter = this.utilities.addFilter(bairro?.datasource, 'SGESTADO', '=', component.value);
        bairro.value = '';
        municipio.datasource.get();
        bairro.datasource.get();
      } else if (component?.name?.includes('CDMUNICIPIO') && component.value) {
        // bairro.datasource.addFilter('CDMUNICIPIO', component.value);
        bairro.datasource.filter = this.utilities.addFilter(bairro?.datasource, 'CDMUNICIPIO', '=', component.value);
        bairro.value = '';
        bairro.datasource.get();
      }
    }
  }
  
  public saveDetail({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    const tabsRota = Metadata.getInstance<Tabs>('tabsRota');
    const activeTab = tabsRota?.tabs[tabsRota.activeTab];
    if (grid?.isGridValid() && activeTab) {
      const gridPai = Metadata.getInstance<GridEditable>('gridRotaEntrega');
      const rows = grid.getEditedRows();
      rows.forEach((el: any) => {
        el.CDROTATRANS = gridPai.datasource.currentRow.CDROTATRANS;
      });
      // const regexFind = /\/find/gm;
      // const datasourceName = (grid.datasource as IDatasource).route.replace(regexFind, '');
      // const datasourceName = 'municipiRota';
      let datasourceName = '';
      if (activeTab.name === 'municipioRota') {
        datasourceName = 'municipiRota';
      } else if (activeTab.name === 'clienteRota') {
        datasourceName = 'clienRota';
        rows.forEach((el: any) => {
          el.NRORDCLIROTA = el.NRORDCLIROTA.padStart(3, '0');
        });
      } else if (activeTab.name === 'unidadeRota') {
        datasourceName = 'filialrota';
        rows.forEach((el: any) => {
          el.CDFILIROTA = el.CDFILIAL;
          el.NRORDFILROTA = el.NRORDFILROTA.padStart(3, '0');
        });
      }
  
      this.crudController.saveRows(rows, datasourceName, grid).then(() => {
        this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  public saveRotaEntrega({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    if (grid?.isGridValid()) {
      const rows = grid.getEditedRows();
      rows.forEach((el: any) => {
        el.CDOPERADOR = localStorage.getItem('CDOPERADOR') || '';
        el.CDROTATRANS = el.CDROTATRANS.padStart(6, '0');
        el.CDFILIAL = this.appController.getCdCentral();
      });
      const regexFind = /\/find/gm;
      const datasourceName = (grid.datasource as IDatasource).route.replace(regexFind, '');
  
      this.crudController.saveRows(rows, datasourceName, grid).then(() => {
        this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  public setFilterTabs({ component }: ITabsBeforeChangeEventParam) {
    const activeTab = component.tabs[component.activeTab];
    if (activeTab?.name === 'municipioRota' && document.getElementsByName('gridMunicipioRota').length > 0) {
      const gridMunicipioRota = Metadata.getInstance<GridEditable>('gridMunicipioRota');
      const gridPai = Metadata.getInstance<GridEditable>('gridRotaEntrega');
      const conditions = [
        {
          name: 'CDROTATRANS',
          operator: '=',
          value: gridPai.datasource.currentRow.CDROTATRANS,
        },
      ];
      
      gridMunicipioRota.datasource.filter = conditions;
      gridMunicipioRota.datasource.get();
    } else if (activeTab?.name === 'clienteRota' && document.getElementsByName('gridClienteRota').length > 0) {
      const gridClienteRota = Metadata.getInstance<GridEditable>('gridClienteRota');
      const gridPai = Metadata.getInstance<GridEditable>('gridRotaEntrega');
      const conditions = [
        {
          name: 'CDROTATRANS',
          operator: '=',
          value: gridPai.datasource.currentRow.CDROTATRANS,
        },
      ];
      
      gridClienteRota.datasource.filter = conditions;
      gridClienteRota.datasource.get();
    } else if (activeTab?.name === 'unidadeRota' && document.getElementsByName('gridUnidadeRota').length > 0) {
      const gridUnidadeRota = Metadata.getInstance<GridEditable>('gridUnidadeRota');
      const gridPai = Metadata.getInstance<GridEditable>('gridRotaEntrega');
      const conditions = [
        {
          name: 'CDROTATRANS',
          operator: '=',
          value: gridPai.datasource.currentRow.CDROTATRANS,
        },
      ];
      
      gridUnidadeRota.datasource.filter = conditions;
      gridUnidadeRota.datasource.get();
    }
  }
}
